#include <stdint.h>

#define SERIAL_PORT_BASE     0xffffc000

#define SERP_RX_ST_REG_o           0x00
#define SERP_RX_ST_REG_READY_m      0x1
#define SERP_RX_ST_REG_IE_m         0x2

#define SERP_RX_DATA_REG_o         0x04

#define SERP_TX_ST_REG_o           0x08
#define SERP_TX_ST_REG_READY_m      0x1
#define SERP_TX_ST_REG_IE_m         0x2

#define SERP_TX_DATA_REG_o         0x0c

static inline void serp_write_reg(uint32_t base, uint32_t reg, uint32_t val)
{
  *(volatile uint32_t *)(base + reg) = val;
}
static inline uint32_t serp_read_reg(uint32_t base, uint32_t reg)
{
  return *(volatile uint32_t *)(base + reg);
}

void serp_tx_byte(int data)
{
  while (!(serp_read_reg(SERIAL_PORT_BASE, SERP_TX_ST_REG_o) &
                SERP_TX_ST_REG_READY_m));
  serp_write_reg(SERIAL_PORT_BASE, SERP_TX_DATA_REG_o, data);
}

int serp_rx_byte()
{
  while (!(serp_read_reg(SERIAL_PORT_BASE, SERP_RX_ST_REG_o) &
                SERP_RX_ST_REG_READY_m));
  return serp_read_reg(SERIAL_PORT_BASE, SERP_RX_DATA_REG_o);
}


int main(void)
{
  const char *text = "Hello world.\nI am alive.\n";
  const char *s;

  for (s = text; *s; s++)
    serp_tx_byte(*s);

  while(1) {
    serp_tx_byte(serp_rx_byte());
  }
  
  return 0;
}
